// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 27th July 2005
//
// Purpose:       Demo shows how to use the ExtractDigits function
//                and for what it can be used.
//
// -----------------------------------------------------------------------------
#include <hel.h>

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define DIGITS_COUNT 8

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
volatile u32 ATTR_MEM_IN_EWRAM g_NewFrame=TRUE;
u32 ATTR_MEM_IN_EWRAM g_Score=0;
u32 ATTR_MEM_IN_EWRAM g_Obj[DIGITS_COUNT];

// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned char digits_Bitmap[640];
extern const unsigned short object_Palette[256];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void InitObjects(void);
void UpdateObjects(void);
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // init objects
    InitObjects();
    
    // load object palette
    hel_PalObjLoad256((void*)object_Palette);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
          // increase score
          g_Score+=1;
          
          // update object graphics
          UpdateObjects();

          g_NewFrame = FALSE;
        }
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Create DIGITS_COUNT objects
// -----------------------------------------------------------------------------
void InitObjects(void)
{
  int i;
  
  for(i=0; i<DIGITS_COUNT; i++)
  {
    g_Obj[i] = hel_ObjCreate256
                (
                  (void*)digits_Bitmap, // Sourcegraphic
                  OBJ_SHAPE_SQUARE,     // Object Shape
                  0,                    // Object Size
                  OBJ_MODE_NORMAL,      // Object Mode
                  10 + i*8,             // X Screen Coordinate
                  10                    // Y Screen Coordinate
                );
  }
}


// -----------------------------------------------------------------------------
// Update Objects, load new graphic into Vram.
// -----------------------------------------------------------------------------
void UpdateObjects(void)
{
  int i;
  u8 DigitsBuffer[DIGITS_COUNT];
  
  
  // Extract the digits from g_Score and store
  // them in DigitsBuffer!
  hel_MathExtractDigits
    (
      g_Score,            // Source Value
      (u8*)DigitsBuffer,  // Destination Buffer
      DIGITS_COUNT        // Count of digits to extract
    );
  
  
  // reload object graphic!
  for(i=0; i<DIGITS_COUNT; i++)
  {
    hel_ObjUpdateGfx
    (
      g_Obj[i],                                     // Obj Handle
      (void*)&digits_Bitmap[8*8*DigitsBuffer[i]]    // Pointer to new graphic, 8*8 = ObjSize and
                                                    // DigitsBuffer[i] contains the current number.
                                                    // Since the digits on the sourcegraphic are stacked,
                                                    // we can just use DigitsBuffer as multiplier to get
                                                    // the correct frame inside the sourcegraphic.
    );
  }
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
  ham_CopyObjToOAM();

  g_NewFrame=TRUE;
}

