#include <hel.h>

#define DrawText(x,y,txt,args...)    ham_DrawText(x,y,txt,##args)
//#define DrawText(x,y,txt,args...)

// column drawer routine for non rotation maps
void MapDrawer_ColumnFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo)
{
  // pDrawInfo->pSourceCurrentDataAddr points to the first maptile in
  // the source mapdata array what the drawing system expects.
  // you dont need to calculate the source address, you can just use
  // this pointer! make sure to cast it to the correct type,
  // u16* in normal mode and u8* in rotation mode!
  u16 *TilePtr = (u16*)pDrawInfo->pSourceCurrentDataAddr;
  
  u32 Y;

  // pDrawInfo->TargetRow contains the index of the first row to update
  // and pDrawInfo->Quantity contains how many maptiles we want to draw.
  for(Y = pDrawInfo->TargetRow; Y<pDrawInfo->TargetRow+pDrawInfo->Quantity; Y++)
  {
    // set the maptile!
    ham_SetMapTile
      (
        pMapInfo->BgNo,           // background number
        pDrawInfo->TargetColumn,  // target column in vram
        Y & 31,                   // target row in vram
        *TilePtr                  // maptile
      );
    
    // move one row further
    TilePtr += pMapInfo->Size.X;
  }
  
  // draw some text
  DrawText(1, 1, "ColumnFunc: %d %d     ", pMapInfo->Position.X, pMapInfo->Position.Y);
}

// row drawer routine for non rotation maps
void MapDrawer_RowFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo)
{
  u16 *TilePtr = (u16*)pDrawInfo->pSourceCurrentDataAddr;
  u32 X;

  for(X = pDrawInfo->TargetColumn; X<pDrawInfo->TargetColumn+pDrawInfo->Quantity; X++)
  {
    ham_SetMapTile(pMapInfo->BgNo, X & 31, pDrawInfo->TargetRow, *TilePtr);
    TilePtr++;
  }
  
  // draw some text
  DrawText(1, 2, "RowFunc...: %d %d      ", pMapInfo->Position.X, pMapInfo->Position.Y);
}


// column drawer routine for rotation maps
void MapDrawer_ColumnRotationFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo)
{
  u8 *TilePtr = (u8*)pDrawInfo->pSourceCurrentDataAddr;
  u32 Y;

  for(Y=pDrawInfo->TargetRow; Y<pDrawInfo->TargetRow+pDrawInfo->Quantity; Y++)
  {
    ham_SetMapTile(pMapInfo->BgNo, pDrawInfo->TargetColumn, Y & 31, *TilePtr);
    TilePtr += pMapInfo->Size.X;
  }
  
  // draw some text
  DrawText(1, 1, "ColumnRotationFunc: %d %d      ", pMapInfo->Position.X, pMapInfo->Position.Y);
}

// row drawer routine for rotation maps
void MapDrawer_RowRotationFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo)
{
  u8 *TilePtr = (u8*)pDrawInfo->pSourceCurrentDataAddr;
  u32 X;

  for(X=pDrawInfo->TargetColumn; X<pDrawInfo->TargetColumn+pDrawInfo->Quantity; X++)
  {
    ham_SetMapTile(pMapInfo->BgNo, X & 31, pDrawInfo->TargetRow, *TilePtr);
    TilePtr++;
  }
  
  // draw some text
  DrawText(1, 2, "RowRotationFunc: %d %d      ", pMapInfo->Position.X, pMapInfo->Position.Y);
}

