// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 15th September 2005
//
// Purpose:       Demo shows how to use the undocumented map-drawer-system.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 g_NewFrame=TRUE;
TMapScrollInfo g_MapInfo ATTR_MEM_IN_EWRAM;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short racetrack_Map[16384];
extern const unsigned short racetrack_Palette[256];
extern const unsigned char racetrack_Tiles[9088];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);

// Implemented in MapDrawer.c
extern void MapDrawer_ColumnFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void MapDrawer_RowFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void MapDrawer_ColumnRotationFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void MapDrawer_RowRotationFunc(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    ham_SetBgMode(0);
    ham_InitText(0);
    ham_SetTextCol(10, 0);

    // Load Tileset into VideoRAM
    ham_bg[1].ti = ham_InitTileSet((void*)&racetrack_Tiles, SIZEOF_16BIT(racetrack_Tiles), 1, 1);

    // Init the Map-System for Background 1
    g_MapInfo = hel_MapInit(1,        // Backgroundnumber
                            1024/8,   // MapWidth in tiles
                            1024/8,   // MapHeight in tiles
                            FALSE,    // RotationMap?
                            (void*)racetrack_Map);   // Source MapData

    // Set our own drawer routines
    g_MapInfo.pRowDrawer = MapDrawer_RowFunc;
    g_MapInfo.pColumnDrawer = MapDrawer_ColumnFunc;
    
    // Load Background Palette
    hel_PalBgLoad256((void*)&racetrack_Palette);

    // Turn on Background
    ham_InitBg(1, TRUE, 0, FALSE);

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);

    // Infinite loop
    for(;;)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // Read current pad state
            hel_PadReadState();

            #define SCROLL_SPEED 3
            TPoint32 ScrollBy={0, 0};

            // Query Pad to see in what direction
            // the map should be scrolled
            if(hel_Pad.Held.Down)
            {
               ScrollBy.Y += SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Up)
            {
               ScrollBy.Y -= SCROLL_SPEED;
            }

            if(hel_Pad.Held.Left)
            {
               ScrollBy.X -= SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Right)
            {
               ScrollBy.X += SCROLL_SPEED;
            }

            // Now Scroll the map!
            hel_MapScrollBy(&g_MapInfo, ScrollBy.X, ScrollBy.Y);

            // Frame done!
            g_NewFrame=FALSE;
        }
    }

    return 0;
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
}

