// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 15th September 2005
//
// Purpose:       Demo shows how to use the Map-System with Parallax Scrolling.
//
// Notes:         Levelgraphics borrowed from Nessie (http://nessie.gbadev.org)
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define RATIO_LAYER_0       PARALLAX_FLOAT_TO_RATIO(1.3f)
#define RATIO_LAYER_1       PARALLAX_FLOAT_TO_RATIO(0.65f)
#define RATIO_LAYER_2       PARALLAX_FLOAT_TO_RATIO(0.2f)
#define SCROLL_SPEED        2


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
volatile u8 ATTR_MEM_IN_EWRAM g_NewFrame=TRUE;
TMapScrollInfo ATTR_MEM_IN_EWRAM g_MapInfos[3];


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short world1_layer0_Map[4096];
extern const unsigned short world1_layer0_Palette[16];
extern const unsigned char world1_layer0_Tiles[896];
extern const unsigned short world1_layer1_Map[2054];
extern const unsigned short world1_layer1_Palette[16];
extern const unsigned char world1_layer1_Tiles[512];
extern const unsigned short world1_layer2_Map[880];
extern const unsigned short world1_layer2_Palette[16];
extern const unsigned char world1_layer2_Tiles[2336];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    ham_SetBgMode(0);
    
    // Load Background Palettes
    hel_PalBgLoad16((void*)world1_layer0_Palette, 0);
    hel_PalBgLoad16((void*)world1_layer1_Palette, 1);
    hel_PalBgLoad16((void*)world1_layer2_Palette, 2);
    
    // Init 16 Color Tilesets
    ham_bg[0].ti = ham_InitTileSet((void*)&world1_layer0_Tiles, SIZEOF_16BIT(world1_layer0_Tiles), 0, 1);
    ham_bg[1].ti = ham_InitTileSet((void*)&world1_layer1_Tiles, SIZEOF_16BIT(world1_layer1_Tiles), 0, 1);
    ham_bg[2].ti = ham_InitTileSet((void*)&world1_layer2_Tiles, SIZEOF_16BIT(world1_layer2_Tiles), 0, 1);

    // Init Maps
    g_MapInfos[0] = hel_MapInit(0, 128, 32, 0, (void*)world1_layer0_Map);
    g_MapInfos[1] = hel_MapInit(1,  79, 26, 0, (void*)world1_layer1_Map);
    g_MapInfos[2] = hel_MapInit(2,  40, 22, 0, (void*)world1_layer2_Map);
    
    // Set Parallax Ratios!
    hel_MapSetParallaxRatio(&g_MapInfos[0], RATIO_LAYER_0, RATIO_LAYER_0);
    hel_MapSetParallaxRatio(&g_MapInfos[1], RATIO_LAYER_1, RATIO_LAYER_1);
    hel_MapSetParallaxRatio(&g_MapInfos[2], RATIO_LAYER_2, RATIO_LAYER_2);
    
    // Jump to the bottom edge of the Map(s)
    hel_MapJumpTo(&g_MapInfos[0], MAP_JUMPTO_BOTTOM);
    hel_MapJumpTo(&g_MapInfos[1], MAP_JUMPTO_BOTTOM);
    hel_MapJumpTo(&g_MapInfos[2], MAP_JUMPTO_BOTTOM);
    
    // Turn on Backgrounds
    ham_InitBg(0, TRUE, 0, FALSE);
    ham_InitBg(1, TRUE, 1, FALSE);
    ham_InitBg(2, TRUE, 2, FALSE);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
            TPoint32 ScrollBy={0,0};
            
            // Read current joypad state
            hel_PadReadState();

            // Get joypad info and scroll around
            if(hel_Pad.Held.Down)
            {
                ScrollBy.Y += SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Up)
            {
                ScrollBy.Y -= SCROLL_SPEED;
            }

            if(hel_Pad.Held.Left)
            {
                ScrollBy.X -= SCROLL_SPEED;
            }
            else
            if(hel_Pad.Held.Right)
            {
                ScrollBy.X += SCROLL_SPEED;
            }


            // Scroll three Map Layers
            hel_MapBatchScrollBy(g_MapInfos, ScrollBy.X, ScrollBy.Y, 3);

            g_NewFrame=FALSE;
        }
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
}

